/**
 * Several utility methods for communicating with the backend.  This is done
 * with either rsh/rcp or ssh/scp.  Methods are based on those defined in
 * OSC's CMUtilites bean, rather than the original submitJob by Tom Haupt.
 */
package WebFlow.submitJob;

import WebFlow.*;
import org.omg.CORBA.*;
import java.io.*;
import java.util.*;

public class submitJobImpl extends WebFlow.BeanContextChildSupport
    implements submitJobOperations {
    
    String msg_;
    org.omg.CORBA.Object peer;
    
    public submitJobImpl(org.omg.CORBA.Object peer, String msg) throws WebFlow.NullPointerException {
	
	super(peer);
	this.peer=peer;
	String msg_=msg;
    }

    //========================================================================
    public void test() {
	System.out.println("SubmitJob2 is up");
    }
    
    //========================================================================
    public String execLocalCommand(String command) {
	// Executes a UNIX "command" locally, on the WebFlow server.
	Runtime rt = Runtime.getRuntime();
	System.err.println("execCommandLocally -- command: "+ command);

	String commandOutput = "COMMAND: " + command+ "\nSTATUS: ";
	try {
	    Process p = rt.exec(command);
	    InputStream in = p.getInputStream();
	    InputStream err = p.getErrorStream();
	    int c;
	    StringBuffer stdoutAndStderr = new StringBuffer("STDOUT:\n");
	    while((c = in.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    in.close();
	    stdoutAndStderr.append("\nSTDERR:\n");
	    while ((c = err.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    err.close();
	    try {
		p.waitFor();
	    }
	    catch (InterruptedException eint) {
		System.err.println(eint.getMessage());
		eint.printStackTrace();
	    }
	commandOutput = commandOutput + p.exitValue()  + "\n"
	    + stdoutAndStderr.toString();
	}
	catch (IOException eio) {
	    System.err.println(eio.getMessage());
	    eio.printStackTrace();
	}
	return commandOutput;
    }

    /**============================================================
     * Executes a UNIX "command" on a remote "host" as "user" using ssh.
     * It is assumed that ssh is set such that no password or 
     * passphrase is needed to execute the command.
     * Return string with command, status, stdout, stderr
     *============================================================*/

    public String execSshCommand(String host, String user, String command) {
	Runtime rt = Runtime.getRuntime();
	String sshCommand = "ssh -l "+user+" "+host+" "+command;
	System.err.println("execCommandOnBackend -- command: "+ sshCommand);

	String commandOutput = "COMMAND: " + sshCommand+ "\nSTATUS: ";
	try {
	    Process p = rt.exec(sshCommand);
	    InputStream in = p.getInputStream();
	    InputStream err = p.getErrorStream();
	    int c;
	    StringBuffer stdoutAndStderr = new StringBuffer("STDOUT:\n");
	    while((c = in.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    in.close();
	    stdoutAndStderr.append("\nSTDERR:\n");
	    while ((c = err.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    err.close();
	    try {
		p.waitFor();
	    }
	    catch (InterruptedException eint) {
		System.err.println(eint.getMessage());
		eint.printStackTrace();
	    }
	commandOutput = commandOutput + p.exitValue()  + "\n"
	    + stdoutAndStderr.toString();
	}
	catch (IOException eio) {
	    System.err.println(eio.getMessage());
	    eio.printStackTrace();
	}
	return commandOutput;
    }

    //========================================================================
    // Executes a UNIX "command" on a remote "host" as "user" using ssh.
    // Return string with command, status, stdout, stderr
    //========================================================================
    public String execRshCommand(String host, String user, String command) {

	Runtime rt = Runtime.getRuntime();
	String sshCommand = "rsh -l "+user+" "+host+" "+command;
	System.err.println("execCommandOnBackend -- command: "+ sshCommand);

	String commandOutput = "COMMAND: " + sshCommand+ "\nSTATUS: ";
	try {
	    Process p = rt.exec(sshCommand);
	    InputStream in = p.getInputStream();
	    InputStream err = p.getErrorStream();
	    int c;
	    StringBuffer stdoutAndStderr = new StringBuffer("STDOUT:\n");
	    while((c = in.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    in.close();
	    stdoutAndStderr.append("\nSTDERR:\n");
	    while ((c = err.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    err.close();
	    try {
		p.waitFor();
	    }
	    catch (InterruptedException eint) {
		System.err.println(eint.getMessage());
		eint.printStackTrace();
	    }
	commandOutput = commandOutput + p.exitValue()  + "\n"
	    + stdoutAndStderr.toString();
	}
	catch (IOException eio) {
	    System.err.println(eio.getMessage());
	    eio.printStackTrace();
	}
	return commandOutput;
    }

    //======================================================
    // This method uses scp to copy files from the remote 
    // host to the WebFlow server.
    // Return string with command, status, stdout, stderr
    //======================================================
    public String scpFileFromBackend(String scpOptions,
				      String user,
				      String host,
				      String remoteFile,
				      String localFile) {


	Runtime rt = Runtime.getRuntime();
        if(scpOptions.length() > 0) {
	    String scpOptionsTrim = scpOptions.trim();
	    if(! scpOptionsTrim.startsWith("-")) {
		scpOptions = "-" + scpOptionsTrim;
	    } 
	}

	String scpCommand = "scp " + scpOptions + " " + user.trim()
	    + "@" + host.trim() +  ":" + remoteFile.trim() + " " +
	    localFile;
	System.err.println("copyFileFromBackend  -- command: "+ scpCommand);

	String commandOutput = "COMMAND: " + scpCommand+ "\nSTATUS: ";
	try {
	    Process p = rt.exec(scpCommand);
	    InputStream in = p.getInputStream();
	    InputStream err = p.getErrorStream();
	    int c;
	    StringBuffer stdoutAndStderr = new StringBuffer("STDOUT:\n");
	    while((c = in.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    in.close();
	    stdoutAndStderr.append("\nSTDERR:\n");
	    while ((c = err.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    err.close();
	    try {
		p.waitFor();
	    }
	    catch (InterruptedException eint) {
		System.err.println(eint.getMessage());
		eint.printStackTrace();
	    }
	commandOutput = commandOutput + p.exitValue()  + "\n"
	    + stdoutAndStderr.toString();
	}
	catch (IOException eio) {
	    System.err.println(eio.getMessage());
	    eio.printStackTrace();
	}
	return commandOutput;
    }
    //======================================================
    // This method uses rcp to copy files from the remote 
    // host to the WebFlow server.
    // Return string with command, status, stdout, stderr
    //======================================================
    public String rcpFileFromBackend(String rcpOptions,
				      String user,
				      String host,
				      String remoteFile,
				      String localFile) {


	Runtime rt = Runtime.getRuntime();
        if(rcpOptions.length() > 0) {
	    String rcpOptionsTrim = rcpOptions.trim();
	    if(! rcpOptionsTrim.startsWith("-")) {
		rcpOptions = "-" + rcpOptionsTrim;
	    } 
	}

	String rcpCommand = "rcp " + rcpOptions + " " + user.trim()
	    + "@" + host.trim() +  ":" + remoteFile.trim() + " " +
	    localFile;
	System.err.println("copyFileFromBackend  -- command: "+ rcpCommand);

	String commandOutput = "COMMAND: " + rcpCommand+ "\nSTATUS: ";
	try {
	    Process p = rt.exec(rcpCommand);
	    InputStream in = p.getInputStream();
	    InputStream err = p.getErrorStream();
	    int c;
	    StringBuffer stdoutAndStderr = new StringBuffer("STDOUT:\n");
	    while((c = in.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    in.close();
	    stdoutAndStderr.append("\nSTDERR:\n");
	    while ((c = err.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    err.close();
	    try {
		p.waitFor();
	    }
	    catch (InterruptedException eint) {
		System.err.println(eint.getMessage());
		eint.printStackTrace();
	    }
	commandOutput = commandOutput + p.exitValue()  + "\n"
	    + stdoutAndStderr.toString();
	}
	catch (IOException eio) {
	    System.err.println(eio.getMessage());
	    eio.printStackTrace();
	}
	return commandOutput;
    }

    //======================================================
    // This method uses scp to copy files from webserver
    // to remote hosts. 
    // Return string with command, status, stdout, stderr
    //======================================================
    public String scpFileToBackend(String scpOptions,
				   String localFile,
				   String user,
				   String host,
				   String remoteFile) {
	
	Runtime rt = Runtime.getRuntime();
        if(scpOptions.length() > 0) {
	    String scpOptionsTrim = scpOptions.trim();
	    if(! scpOptionsTrim.startsWith("-")) {
		scpOptions = "-" + scpOptionsTrim;
	    } 
	}

	String scpCommand = "scp " + scpOptions + " " + localFile + " " +
	    user.trim() + "@" + host.trim() +  ":" + remoteFile.trim();
	System.err.println("copyFileToBackend  -- command: "+ scpCommand);

	String commandOutput = "COMMAND: " + scpCommand+ "\nSTATUS: ";
	try {
	    Process p = rt.exec(scpCommand);
	    InputStream in = p.getInputStream();
	    InputStream err = p.getErrorStream();
	    int c;
	    StringBuffer stdoutAndStderr = new StringBuffer("STDOUT:\n");
	    while((c = in.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    in.close();
	    stdoutAndStderr.append("\nSTDERR:\n");
	    while ((c = err.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    err.close();
	    try {
		p.waitFor();
	    }
	    catch (InterruptedException eint) {
		System.err.println(eint.getMessage());
		eint.printStackTrace();
	    }
	commandOutput = commandOutput + p.exitValue()  + "\n"
	    + stdoutAndStderr.toString();
	}
	catch (IOException eio) {
	    System.err.println(eio.getMessage());
	    eio.printStackTrace();
	}
	return commandOutput;
    }

    //======================================================
    // This method uses rcp to copy files from webserver
    // to remote hosts. 
    // Return string with command, status, stdout, stderr
    //======================================================
    public String rcpFileToBackend(String rcpOptions,
				   String localFile,
				   String user,
				   String host,
				   String remoteFile) {
	
	Runtime rt = Runtime.getRuntime();
        if(rcpOptions.length() > 0) {
	    String rcpOptionsTrim = rcpOptions.trim();
	    if(! rcpOptionsTrim.startsWith("-")) {
		rcpOptions = "-" + rcpOptionsTrim;
	    } 
	}

	String rcpCommand = "rcp " + rcpOptions + " " + localFile + " " +
	    user.trim() + "@" + host.trim() +  ":" + remoteFile.trim();
	System.err.println("copyFileToBackend  -- command: "+ rcpCommand);

	String commandOutput = "COMMAND: " + rcpCommand+ "\nSTATUS: ";
	try {
	    Process p = rt.exec(rcpCommand);
	    InputStream in = p.getInputStream();
	    InputStream err = p.getErrorStream();
	    int c;
	    StringBuffer stdoutAndStderr = new StringBuffer("STDOUT:\n");
	    while((c = in.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    in.close();
	    stdoutAndStderr.append("\nSTDERR:\n");
	    while ((c = err.read()) != -1) {
		stdoutAndStderr.append((char)c);
	    }
	    err.close();
	    try {
		p.waitFor();
	    }
	    catch (InterruptedException eint) {
		System.err.println(eint.getMessage());
		eint.printStackTrace();
	    }
	commandOutput = commandOutput + p.exitValue()  + "\n"
	    + stdoutAndStderr.toString();
	}
	catch (IOException eio) {
	    System.err.println(eio.getMessage());
	    eio.printStackTrace();
	}
	return commandOutput;
    }

}
